@@ -20,73 +20,57 @@ class ImputedData:
     Parameters
     ----------
     impute_data: pandas DataFrame or np.ndarray
-
-        .. code-block:: text
-
-            The data used by the kernel to impute impute_data.
+        The data used by the kernel to impute impute_data.
 
     variable_schema: None or ist or dict, default=None
-
-        .. code-block:: text
-
-            - If None all variables are used to impute all variables which have missing values.
-            - If list all columns in data are used to impute the variables in the list
-            - If dict the values will be used to impute the keys. Either can be column
-                indices or names (if data is a pd.DataFrame).
+        If None all variables are used to impute all variables which have
+        missing values.
+        If list all columns in data are used to impute the variables in the list
+        If dict the values will be used to impute the keys. Either can be column
+        indices or names (if data is a pd.DataFrame).
 
     imputation_order: str, list[str], list[int], default="ascending"
+        The order the imputations should occur in.
+            ascending: variables are imputed from least to most missing
+            descending: most to least missing
+            roman: from left to right in the dataset
+            arabic: from right to left in the dataset.
 
-        .. code-block:: text
-
-            The order the imputations should occur in.
-                ascending: variables are imputed from least to most missing
-                descending: most to least missing
-                roman: from left to right in the dataset
-                arabic: from right to left in the dataset.
-
-            If a list is provided, the variables will be imputed in that order.
+        If a list is provided, the variables will be imputed in that order.
 
     categorical_feature: str or list, default="auto"
+        The categorical features in the dataset. This handling depends on class of impute_data:
 
-        .. code-block:: text
+            pandas DataFrame:
+                - "auto": categorical information is inferred from any columns with
+                    datatype category or object.
+                - list of column names (or indices): Useful if all categorical columns
+                    have already been cast to numeric encodings of some type, otherwise you
+                    should just use "auto". Will throw an error if a list is provided AND
+                    categorical dtypes exist in data. If a list is provided, values in the
+                    columns must be consecutive integers starting at 0, as required by lightgbm.
 
-            The categorical features in the dataset. This handling depends on class of impute_data:
-                pandas DataFrame:
-                    - "auto": categorical information is inferred from any columns with
-                        datatype category or object.
-                    - list of column names (or indices): Useful if all categorical columns
-                        have already been cast to numeric encodings of some type, otherwise you
-                        should just use "auto". Will throw an error if a list is provided AND
-                        categorical dtypes exist in data. If a list is provided, values in the
-                        columns must be consecutive integers starting at 0, as required by lightgbm.
-
-                numpy ndarray:
-                    - "auto": no categorical information is stored.
-                    - list of column indices: Specified columns are treated as categorical. Column
-                        values must be consecutive integers starting at 0, as required by lightgbm.
+            numpy ndarray:
+                - "auto": no categorical information is stored.
+                - list of column indices: Specified columns are treated as categorical. Column
+                    values must be consecutive integers starting at 0, as required by lightgbm.
 
     save_all_iterations: boolean, default=True
-
-        .. code-block:: text
-
-            Save all the imputation values from all iterations, or just
-            the latest. Saving all iterations allows for additional
-            plotting, but may take more memory
+        Save all the imputation values from all iterations, or just
+        the latest. Saving all iterations allows for additional
+        plotting, but may take more memory
 
     copy_data: boolean, default=True
+        Should the dataset be referenced directly? If False, this will cause the dataset to be
+        altered in place. If a copy is created, it is saved in self.working_data. There are different
+        ways in which the dataset can be altered:
 
-        .. code-block:: text
-
-            Should the dataset be referenced directly? If False, this will cause the dataset to be
-            altered in place. If a copy is created, it is saved in self.working_data. There are different
-            ways in which the dataset can be altered:
-
-            1) complete_data() will fill in missing values
-            2) To save space, mice() references and manipulates self.working_data directly.
-                If self.working_data is a reference to the original dataset, the original
-                dataset will undergo these manipulations during the mice process.
-                At the end of the mice process, missing will be set back to np.NaN where
-                they were originally missing.
+        1) complete_data() will fill in missing values
+        2) To save space, mice() references and manipulates self.working_data directly.
+            If self.working_data is a reference to the original dataset, the original
+            dataset will undergo these manipulations during the mice process.
+            At the end of the mice process, missing will be set back to np.NaN where
+            they were originally missing.
 
     """
 
@@ -96,7 +80,6 @@ class ImputedData:
         datasets=5,
         variable_schema=None,
         imputation_order="ascending",
-        train_nonmissing=False,
         categorical_feature="auto",
         save_all_iterations=True,
         copy_data=True,
@@ -160,8 +143,6 @@ class ImputedData:
             vars_with_any_missing = [
                 int(col) for col, ind in na_where.items() if len(ind > 0)
             ]
-            if len(vars_with_any_missing) == 0:
-                raise ValueError("No missing values to impute.")
 
         elif isinstance(self.working_data, np.ndarray):
 
@@ -210,13 +191,12 @@ class ImputedData:
 
             raise ValueError("impute_data not recognized.")
 
-        self.column_names = column_names
-
         # Formatting of variable_schema.
         if variable_schema is None:
+            variable_schema = vars_with_any_missing
             variable_schema = {
                 var: list(np.setdiff1d(range(data_shape[1]), [var]))
-                for var in np.arange(data_shape[1])
+                for var in variable_schema
             }
         else:
             if isinstance(variable_schema, list):
@@ -246,17 +226,21 @@ class ImputedData:
                         + " variables cannot be used to impute itself."
                     )
 
+            not_imputable = [
+                var for var in list(variable_schema) if var not in vars_with_any_missing
+            ]
+            for rnm in not_imputable:
+                del variable_schema[rnm]
+
         # Format imputation order
         if isinstance(imputation_order, list):
-            imputation_order = self._get_variable_index(imputation_order)
-            assert set(imputation_order).issubset(
-                variable_schema
-            ), "variable_schema does not include all variables to be imputed."
-            imputation_order = [
-                i
-                for i in self._get_variable_index(imputation_order)
-                if na_counts[i] > 0
-            ]
+            # subset because response vars can be removed if imputing new data with no missing values.
+            assert set(variable_schema).issubset(
+                imputation_order
+            ), "imputation_order does not include all variables to be imputed."
+            imputation_order = self._get_variable_index(
+                [i for i in imputation_order if i in list(variable_schema)]
+            )
         elif isinstance(imputation_order, str):
             if imputation_order in ["ascending", "descending"]:
                 imputation_order = (
@@ -265,54 +249,42 @@ class ImputedData:
                     else np.argsort(na_counts)[::-1]
                 )
                 imputation_order = [
-                    int(i)
-                    for i in imputation_order
-                    if na_counts[i] > 0 and i in list(variable_schema)
+                    int(i) for i in imputation_order if i in list(variable_schema)
                 ]
             elif imputation_order == "roman":
-                imputation_order = list(variable_schema).copy()
+                imputation_order = list(variable_schema)
             elif imputation_order == "arabic":
-                imputation_order = list(variable_schema).copy()
+                imputation_order = list(variable_schema)
                 imputation_order.reverse()
             else:
                 raise ValueError("imputation_order not recognized.")
 
-        self.variable_schema = variable_schema
-        self.imputation_order = list(imputation_order)
-        self.unimputed_variables = list(
-            np.setdiff1d(np.arange(data_shape[1]), imputation_order)
-        )
-        if train_nonmissing:
-            self.variable_training_order = [
-                v
-                for v in self.imputation_order + self.unimputed_variables
-                if v in list(self.variable_schema)
-            ]
-        else:
-            self.variable_training_order = self.imputation_order
+        # Get distinct predictor variables
         predictor_vars = [prd for prd in variable_schema.values()]
         self.predictor_vars = list(
             dict.fromkeys([item for sublist in predictor_vars for item in sublist])
         )
-        self.categorical_feature = categorical_feature
+
+        self.column_names = column_names
         self.categorical_variables = categorical_variables
         self.category_counts = category_counts
         self.original_data_class = original_data_class
         self.save_all_iterations = save_all_iterations
+        self.variable_schema = variable_schema
+        self.imputation_order = imputation_order
         self.data_shape = data_shape
         self.na_counts = na_counts
         self.na_where = na_where
         self.vars_with_any_missing = vars_with_any_missing
         self.imputed_variable_count = len(imputation_order)
-        self.modeled_variable_count = len(self.variable_training_order)
         self.iterations = np.zeros(
-            shape=(datasets, self.modeled_variable_count)
+            shape=(datasets, self.imputed_variable_count)
         ).astype(int)
 
         # Create structure to store imputation values.
         # These will be initialized by an ImputationKernel.
         self.imputation_values = {
-            ds: {var: dict() for var in self.imputation_order} for ds in range(datasets)
+            ds: {var: dict() for var in list(variable_schema)} for ds in range(datasets)
         }
         self.initialized = False
 
@@ -351,7 +323,7 @@ save_all_iterations: {self.save_all_iterations}"""
         Datasets are defined by how many different sets of imputation
         values we have accumulated.
         """
-        return self.iterations.shape[0]
+        return len(self.imputation_values)
 
     def _get_variable_name(self, ind):
         """Gets the variable name from an index."""
@@ -389,9 +361,7 @@ save_all_iterations: {self.save_all_iterations}"""
         return indx
 
     def _get_working_data_nonmissing_indx(self, var):
-        non_missing_ind = np.setdiff1d(
-            np.arange(self.data_shape[0]), self.na_where[var]
-        )
+        non_missing_ind = np.setdiff1d(range(self.data_shape[0]), self.na_where[var])
         return non_missing_ind
 
     def _insert_new_data(self, dataset, variable_index, new_data):
@@ -474,14 +444,14 @@ save_all_iterations: {self.save_all_iterations}"""
         )
 
         if variables is None:
-            var = self.variable_training_order
+            var = self.imputation_order
         else:
             variables = _ensure_iterable(variables)
             var = self._get_variable_index(variables)
 
-        assert set(var).issubset(self.variable_training_order)
+        assert set(var).issubset(self.imputation_order)
 
-        iter_indx = [self.variable_training_order.index(v) for v in var]
+        iter_indx = [self.imputation_order.index(v) for v in var]
         ds_uniq = np.unique(self.iterations[np.ix_(ds, iter_indx)])
         if len(ds_uniq) > 1:
             raise ValueError(
@@ -696,7 +666,7 @@ save_all_iterations: {self.save_all_iterations}"""
         # at each iteration
         correlation_dict = {}
         if self.save_all_iterations:
-            iter_range = list(range(1, curr_iteration + 1))
+            iter_range = list(range(curr_iteration + 1))
         else:
             # Make this iterable for code tidyness
             iter_range = [curr_iteration]
@@ -763,7 +733,7 @@ save_all_iterations: {self.save_all_iterations}"""
                 list(correlation_dict[var].values()),
                 labels=range(len(correlation_dict[var])),
             )
-            ax[axr, axc].set_title(self._get_variable_name(var))
+            ax[axr, axc].set_title(var)
             ax[axr, axc].set_xlabel("Iteration")
             ax[axr, axc].set_ylabel("Correlations")
             ax[axr, axc].set_ylim([-1, 1])
