@@ -6,6 +6,7 @@ from .utils import ensure_rng, MeanMatchType, VarSchemType, VarParamType
 from .logger import Logger
 import numpy as np
 from .compat import pd_DataFrame, pd_Series
+import warnings
 
 
 class MultipleImputedKernel(MultipleImputedDataSet):
@@ -156,6 +157,12 @@ class MultipleImputedKernel(MultipleImputedDataSet):
         random_state: Union[int, np.random.RandomState] = None,
     ):
 
+        warnings.warn(
+            "MultipleImputedKernel is depreciated as of version 5.0.0 in favor "
+            + "of the more efficient ImputationKernel, and will be removed in a "
+            + "future release."
+        )
+
         self.random_state = ensure_rng(random_state)
 
         super().__init__(
@@ -168,21 +175,22 @@ class MultipleImputedKernel(MultipleImputedDataSet):
 
         # Prime with the required number of kernel datasets
         while self.dataset_count() < datasets:
-            self.append(
-                KernelDataSet(
-                    data=data,
-                    variable_schema=variable_schema,
-                    mean_match_candidates=mean_match_candidates,
-                    mean_match_subset=mean_match_subset,
-                    mean_match_function=mean_match_function,
-                    imputation_order=imputation_order,
-                    categorical_feature=categorical_feature,
-                    initialization=initialization,
-                    save_all_iterations=save_all_iterations,
-                    save_models=save_models,
-                    random_state=self.random_state,
-                )
+            kds = KernelDataSet(
+                data=data,
+                variable_schema=variable_schema,
+                mean_match_candidates=mean_match_candidates,
+                mean_match_subset=mean_match_subset,
+                mean_match_function=mean_match_function,
+                imputation_order=imputation_order,
+                categorical_feature=categorical_feature,
+                initialization=initialization,
+                save_all_iterations=save_all_iterations,
+                save_models=save_models,
+                random_state=self.random_state,
             )
+            kds.data = self.data
+            kds.na_where = self.na_where
+            self.append(kds)
 
         self.save_models = save_models
         self.mean_match_candidates = getattr(self[0], "mean_match_candidates")
